unit QSearch;

interface
uses
  SysUtils, {$IFDEF WIN32} Windows, Registry, {$ELSE} WinTypes, WinProcs, {$ENDIF}
   Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, ExtCtrls, DB, DBTables;

type
  EDBFindError = class(Exception)
                 end;

  T_SearchDlg = class(TForm)
    pnlSearch: TPanel;
    labSearchStr: TLabel;
    procedure FormKeyPress(Sender: TObject; var Key: Char);
  private
    FDataSet : TDataSet;
    FKeyField : String;
  public
    { Public declarations }
    FFindText : String;
    function SearchBy(const S : string) : boolean;
  end;

  TQuickSearch = class(TComponent)
  private
    FDataSet : TDataSet;
    FKeyField : String;
    dlgSearch: T_SearchDlg;

    procedure SetDataSet (Value : TDataSet);
    Procedure SetKeyField (Value : String);
  protected
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent);override;
    destructor Destroy;override;
    procedure Execute(Value : String);
    { Public declarations }
  published
    property DataSet : TDataSet
      read FDataSet
      write SetDataSet;
    property KeyField : String
      read FKeyField
      write SetKeyField;
  end;

procedure Register;

implementation

{$R *.DFM}

procedure Register;
begin
  RegisterComponents('DataControl', [TQuickSearch]);
end;

constructor TQuickSearch.Create(AOwner : TComponent);
begin
 inherited Create(AOwner);
 dlgSearch := T_SearchDlg.Create(nil);
end;

destructor TQuickSearch.Destroy;
begin
 dlgSearch.Free;
 inherited Destroy;
end;

procedure TQuickSearch.SetDataSet (Value : TDataSet);
begin
{$IFNDEF WIN32}
  if not(Value is TTable) then
    Raise EDBFindError.Create('DataSet must be a TTable');
{$ENDIF}
  if Value <> FDataSet then FDataSet := Value;
  dlgSearch.FDataSet := FDataSet;
end;

procedure TQuickSearch.SetKeyField (Value : String);
var
  i : Integer;
  Ok : Boolean;
begin
  if FDataSet <> nil
  then begin
   if FDataSet.FindField('Value') = nil
   then EDBFindError.Create('Can''t find field "'+Value+'"');
  end;
  FKeyField := Value;
  dlgSearch.FKeyField := FKeyField;
end;

procedure TQuickSearch.Execute(Value : String);
var
 i : Smallint;
begin
  dlgSearch.FFindText := '';
  dlgSearch.labSearchStr.Caption := '';
  for i := Length(Value) downto 1 do
  begin
   if dlgSearch.SearchBy(Value)
   then begin
    dlgSearch.FFindText := Value;
    dlgSearch.labSearchStr.Caption := dlgSearch.FFindText;
    Break;
   end
   else Delete(Value,Length(Value),1);
  end;
  dlgSearch.ShowModal;
end;

procedure T_SearchDlg.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if (Key = #27) or (Key = #13) then Close
  else if (Key = #8) and(length(FFindText) > 0)
  then begin
   delete(FFindText,length(FFindText),1);
   labSearchStr.Caption := FFindText;
   SearchBy(FFindText);
  end
  else if Key > #31
  then begin
   if SearchBy(FFindText+Key)
   then begin
    FFindText := FFindText + Key;
    labSearchStr.Caption := FFindText
   end;
  end
end;

function T_SearchDlg.SearchBy(const S : string) : boolean;
var
 OldKey : String;
 Bmrk : TBookmark;
begin
 Result := false;
 Bmrk := FDataSet.GetBookmark;
 try
  Screen.Cursor := crHourGlass;
{$IFDEF WIN32}
  Result := FDataSet.Locate(FKeyField,S,[loPartialKey,loCaseInsensitive]);
  if not Result then FDataSet.GotoBookmark(Bmrk);
{$ELSE}
  OldKey := (FDataSet as TTable).IndexFieldNames;
  (FDataSet as TTable).IndexFieldNames := FKeyField;
  (FDataSet as TTable).FindNearest([S]);
  if Pos(S,FDataSet.FieldByName(FKeyField).AsString) = 1
  then Result := true
  else FDataSet.GotoBookmark(Bmrk);
  (FDataSet as TTable).IndexFieldNames := OldKey;
{$ENDIF}
 finally
  Screen.Cursor := crDefault;
  FDataSet.FreeBookmark(Bmrk);
 end;
end;

END.
